@@ -19,6 +19,28 @@ public class TagSelectionWidget : TreeView {
 	// Hash of the IDs of the selected tags.
 	private Hashtable selection;
 
+	public Tag TagAtPosition (int x, int y) 
+	{
+		TreeModel model;
+		TreePath path;
+		TreeIter iter;
+
+		// Work out which tag we're dropping onto
+		if (!this.GetPathAtPos (x, y, out path))
+			return null;
+
+		if (!Model.GetIter (out iter, path))
+			return null;
+
+		GLib.Value value = new GLib.Value ();
+
+		Model.GetValue (iter, 0, ref value);
+		uint tag_id = (uint) value;
+		Tag tag = tag_store.Get (tag_id) as Tag;
+
+		return tag;
+	}
+
 	public void Select (Tag tag)
 	{
 		if (! selection.Contains (tag.Id))
@@ -77,6 +99,8 @@ public class TagSelectionWidget : TreeView {
 			foreach (Tag t in value)
 				Select (t);
 
+			//FIXME this should really just toggle the items not rebuild the list
+			Update ();
 			SelectionChanged (this);
 		}
 	}
@@ -107,7 +131,7 @@ public class TagSelectionWidget : TreeView {
 		foreach (TreePath path in rows) {
 			GLib.Value value = new GLib.Value ();
 			Model.GetIter (out iter, path);
-			Model.GetValue (iter, 0, value);
+			Model.GetValue (iter, 0, ref value);
 			uint tag_id = (uint) value;
 			tags[i] = tag_store.Get (tag_id) as Tag;
 			i++;
@@ -141,7 +165,7 @@ public class TagSelectionWidget : TreeView {
 		Model.GetIter (out iter, path);
 
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, value);
+		Model.GetValue (iter, 0, ref value);
 		uint tag_id = (uint) value;
 		Tag tag = tag_store.Get (tag_id) as Tag;
 
@@ -183,7 +207,7 @@ public class TagSelectionWidget : TreeView {
 				       TreeIter iter)
 	{
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, value);
+		Model.GetValue (iter, 0, ref value);
 		uint tag_id = (uint) value;
 		Tag tag = tag_store.Get (tag_id) as Tag;
 
@@ -197,7 +221,7 @@ public class TagSelectionWidget : TreeView {
 				   TreeIter iter)
 	{
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, value);
+		Model.GetValue (iter, 0, ref value);
 		uint tag_id = (uint) value;
 		Tag tag = tag_store.Get (tag_id) as Tag;
 
@@ -220,7 +244,7 @@ public class TagSelectionWidget : TreeView {
 			return;
 
 		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, 0, value);
+		Model.GetValue (iter, 0, ref value);
 		uint tag_id = (uint) value;
 
 		Tag tag = tag_store.Get (tag_id) as Tag;
